@@ -1,7 +1,7 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 local client = client
@@ -11,7 +11,7 @@ local atag = require("awful.tag")
 
 --- When loaded, this module makes sure that there's always a client that will have focus
 -- on event such as tag switching, client unmanaging, etc.
-module("awful.autofocus")
+-- awful.autofocus
 
 -- Give focus when clients appear/disappear.
 -- @param obj An object that should have a .screen property.
@@ -24,23 +24,22 @@ local function check_focus(obj)
 end
 
 -- Give focus on tag selection change.
--- @param obj An object that should have a .screen property.
-local function check_focus_screen(obj)
-    check_focus(obj)
-    if client.focus and client.focus.screen ~= obj.screen then
-        local c = nil
-        c = aclient.focus.history.get(obj.screen, 0)
+-- @param tag A tag object
+local function check_focus_tag(t)
+    local s = atag.getscreen(t)
+    if not s then return end
+    check_focus({ screen = s })
+    if client.focus and client.focus.screen ~= s then
+        local c = aclient.focus.history.get(s, 0)
         if c then client.focus = c end
     end
 end
 
-atag.attached_add_signal(nil, "property::selected", check_focus_screen)
-client.add_signal("unmanage", check_focus)
-client.add_signal("new", function(c)
-    c:add_signal("tagged", check_focus)
-    c:add_signal("untagged", check_focus)
-    c:add_signal("property::hidden", check_focus)
-    c:add_signal("property::minimized", check_focus)
-end)
+atag.attached_connect_signal(nil, "property::selected", check_focus_tag)
+client.connect_signal("unmanage", check_focus)
+client.connect_signal("tagged", check_focus)
+client.connect_signal("untagged", check_focus)
+client.connect_signal("property::hidden", check_focus)
+client.connect_signal("property::minimized", check_focus)
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
