@@ -2,7 +2,7 @@
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @author Sébastien Gross &lt;seb-awesome@chezwam.org&gt;
 -- @copyright 2008 Julien Danjou, Sébastien Gross
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -12,11 +12,13 @@ local table = table
 local math = math
 local print = print
 local pairs = pairs
+local string = string
 local util = require("awful.util")
 
 --- Completion module.
 -- This module store a set of function using shell to complete commands name.
-module("awful.completion")
+-- awful.completion
+local completion = {}
 
 -- mapping of command/completion function
 local bashcomp_funcs = {}
@@ -25,7 +27,7 @@ local bashcomp_src = "/etc/bash_completion"
 --- Enable programmable bash completion in awful.completion.bash at the price of
 -- a slight overhead.
 -- @param src The bash completion source file, /etc/bash_completion by default.
-function bashcomp_load(src)
+function completion.bashcomp_load(src)
     if src then bashcomp_src = src end
     local c, err = io.popen("/usr/bin/env bash -c 'source " .. bashcomp_src .. "; complete -p'")
     if c then
@@ -58,7 +60,7 @@ end
 -- @param ncomp The element number to complete.
 -- @param shell The shell to use for completion (bash (default) or zsh).
 -- @return The new command, the new cursor position, the table of all matches.
-function shell(command, cur_pos, ncomp, shell)
+function completion.shell(command, cur_pos, ncomp, shell)
     local wstart = 1
     local wend = 1
     local words = {}
@@ -88,21 +90,22 @@ function shell(command, cur_pos, ncomp, shell)
         i = i + 1
     end
 
-    if cword_index == 1 then
+    if cword_index == 1 and not string.find(words[cword_index], "/") then
         comptype = "command"
     end
 
     local shell_cmd
     if shell == "zsh" or (not shell and os.getenv("SHELL"):match("zsh$")) then
         if comptype == "file" then
-            shell_cmd = "/usr/bin/env zsh -c 'local -a res; res=( " .. words[cword_index] .. "* ); print -l -- ${res[@]}'"
+            shell_cmd = "/usr/bin/env zsh -c 'local -a res; res=( " .. words[cword_index] .. "* ); print -ln -- ${res[@]}'"
         else
             -- check commands, aliases, builtins, functions and reswords
             shell_cmd = "/usr/bin/env zsh -c 'local -a res; "..
             "res=( "..
             "\"${(k)commands[@]}\" \"${(k)aliases[@]}\" \"${(k)builtins[@]}\" \"${(k)functions[@]}\" \"${(k)reswords[@]}\" "..
+            "${PWD}/*(:t)"..
             "); "..
-            "print -l -- ${(M)res[@]:#"..words[cword_index].."*}'"
+            "print -ln -- ${(M)res[@]:#"..words[cword_index].."*}'"
         end
     else
         if bashcomp_funcs[words[1]] then
@@ -158,7 +161,7 @@ end
 -- @param ncomp The number of yet requested completion using current text.
 -- @param keywords The keywords table uised for completion.
 -- @return The new match, the new cursor position, the table of all matches.
-function generic(text, cur_pos, ncomp, keywords)
+function completion.generic(text, cur_pos, ncomp, keywords)
     -- The keywords table may be empty
     if #keywords == 0 then
         return text, #text + 1
@@ -167,7 +170,7 @@ function generic(text, cur_pos, ncomp, keywords)
     -- if no text had been typed yet, then we could start cycling around all
     -- keywords with out filtering and move the cursor at the end of keyword
     if text == nil or #text == 0 then
-        ncomp = math.mod(ncomp - 1, #keywords) + 1
+        ncomp = math.fmod(ncomp - 1, #keywords) + 1
         return keywords[ncomp], #keywords[ncomp] + 2
     end
 
@@ -185,8 +188,10 @@ function generic(text, cur_pos, ncomp, keywords)
     end
 
     --  cycle around all matches
-    ncomp = math.mod(ncomp - 1, #matches) + 1
+    ncomp = math.fmod(ncomp - 1, #matches) + 1
     return matches[ncomp], #matches[ncomp] + 1, matches
 end
 
+return completion
+
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
