@@ -1,7 +1,7 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -10,7 +10,7 @@ local dbus = dbus
 --- D-Bus module for awful.
 -- This module simply request the org.naquadah.awesome.awful name on the D-Bus
 -- for futur usage by other awful modules.
-module("awful.dbus")
+-- awful.dbus
 
 if dbus then
     dbus.request_name("session", "org.naquadah.awesome.awful")
