@@ -1,13 +1,16 @@
 ---------------------------------------------------------------------------
 -- @author Gregor Best
 -- @copyright 2008 Gregor Best
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 --- Dummy function for floating layout
-module("awful.layout.suit.floating")
+-- awful.layout.suit.floating
+local floating = {}
 
-function arrange()
+function floating.arrange()
 end
 
-name = "floating"
+floating.name = "floating"
+
+return floating
