@@ -1,7 +1,7 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2008 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -16,9 +16,10 @@ local capi =
 local client = require("awful.client")
 
 --- Magnifier layout module for awful
-module("awful.layout.suit.magnifier")
+-- awful.layout.suit.magnifier
+local magnifier = {}
 
-function arrange(p)
+function magnifier.arrange(p)
     -- Fullscreen?
     local area = p.workarea
     local cls = p.clients
@@ -55,7 +56,14 @@ function arrange(p)
         geometry.width = area.width
         geometry.height = area.height
     end
-    focus:geometry(geometry)
+
+    local g = {
+        x = geometry.x,
+        y = geometry.y,
+        width = geometry.width - focus.border_width * 2,
+        height = geometry.height - focus.border_width * 2
+    }
+    focus:geometry(g)
     focus:raise()
 
     if #cls > 1 then
@@ -83,10 +91,18 @@ function arrange(p)
         -- Then move clients that are after focused client.
         -- So the next focused window will be the one at the top of the screen.
         for k = 1, fidx - 1 do
-            cls[k]:geometry(geometry)
+            local g = {
+                x = geometry.x,
+                y = geometry.y,
+                width = geometry.width - cls[k].border_width * 2,
+                height = geometry.height - cls[k].border_width * 2
+            }
+            cls[k]:geometry(g)
             geometry.y = geometry.y + geometry.height
         end
     end
 end
 
-name = "magnifier"
+magnifier.name = "magnifier"
+
+return magnifier
