@@ -1,7 +1,7 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2008 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -9,7 +9,8 @@ local pairs = pairs
 local client = require("awful.client")
 
 --- Maximized and fullscreen layouts module for awful
-module("awful.layout.suit.max")
+-- awful.layout.suit.max
+local max = {}
 
 local function fmax(p, fs)
     -- Fullscreen?
@@ -21,21 +22,29 @@ local function fmax(p, fs)
     end
 
     for k, c in pairs(p.clients) do
-        c:geometry(area)
+        local g = {
+            x = area.x,
+            y = area.y,
+            width = area.width - c.border_width * 2,
+            height = area.height - c.border_width * 2
+        }
+        c:geometry(g)
     end
 end
 
 --- Maximized layout.
 -- @param screen The screen to arrange.
-name = "max"
-function arrange(p)
+max.name = "max"
+function max.arrange(p)
     return fmax(p, false)
 end
 
 --- Fullscreen layout.
 -- @param screen The screen to arrange.
-fullscreen = {}
-fullscreen.name = "fullscreen"
-function fullscreen.arrange(p)
+max.fullscreen = {}
+max.fullscreen.name = "fullscreen"
+function max.fullscreen.arrange(p)
     return fmax(p, true)
 end
+
+return max
