@@ -3,7 +3,7 @@
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Donald Ephraim Curtis
 -- @copyright 2008 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -12,7 +12,8 @@ local math = math
 local tag = require("awful.tag")
 
 --- Tiled layouts module for awful
-module("awful.layout.suit.tile")
+-- awful.layout.suit.tile
+local tile = {}
 
 local function tile_group(cls, wa, orientation, fact, group)
     -- get our orientation right
@@ -28,7 +29,7 @@ local function tile_group(cls, wa, orientation, fact, group)
     end
 
     -- make this more generic (not just width)
-    available = wa[width] - (group.coord - wa[x])
+    local available = wa[width] - (group.coord - wa[x])
 
     -- find our total values
     local total_fact = 0
@@ -58,21 +59,21 @@ local function tile_group(cls, wa, orientation, fact, group)
     local unused = wa[height]
     for c = group.first,group.last do
         local i = c - group.first +1
-        geom[width] = size
-        geom[height] = math.floor(unused * fact[i] / total_fact)
+        geom[width] = size - cls[c].border_width * 2
+        geom[height] = math.floor(unused * fact[i] / total_fact) - cls[c].border_width * 2
         geom[x] = group.coord
         geom[y] = coord
         geom = cls[c]:geometry(geom)
-        coord = coord + geom[height]
-        unused = unused - geom[height]
+        coord = coord + geom[height] + cls[c].border_width * 2
+        unused = unused - geom[height] - cls[c].border_width * 2
         total_fact = total_fact - fact[i]
-        used_size = math.max(used_size, geom[width])
+        used_size = math.max(used_size, geom[width] + cls[c].border_width * 2)
     end
 
     return used_size
 end
 
-local function tile(param, orientation)
+local function do_tile(param, orientation)
     local t = tag.selected(param.screen)
     orientation = orientation or "right"
 
@@ -148,33 +149,35 @@ local function tile(param, orientation)
 
 end
 
-right = {}
-right.name = "tile"
-right.arrange = tile
+tile.right = {}
+tile.right.name = "tile"
+tile.right.arrange = do_tile
 
 --- The main tile algo, on left.
 -- @param screen The screen number to tile.
-left = {}
-left.name = "tileleft"
-function left.arrange(p)
-    return tile(p, "left")
+tile.left = {}
+tile.left.name = "tileleft"
+function tile.left.arrange(p)
+    return do_tile(p, "left")
 end
 
 --- The main tile algo, on bottom.
 -- @param screen The screen number to tile.
-bottom = {}
-bottom.name = "tilebottom"
-function bottom.arrange(p)
-    return tile(p, "bottom")
+tile.bottom = {}
+tile.bottom.name = "tilebottom"
+function tile.bottom.arrange(p)
+    return do_tile(p, "bottom")
 end
 
 --- The main tile algo, on top.
 -- @param screen The screen number to tile.
-top = {}
-top.name = "tiletop"
-function top.arrange(p)
-    return tile(p, "top")
+tile.top = {}
+tile.top.name = "tiletop"
+function tile.top.arrange(p)
+    return do_tile(p, "top")
 end
 
-arrange = right.arrange
-name = right.name
+tile.arrange = tile.right.arrange
+tile.name = tile.right.name
+
+return tile
