@@ -1,13 +1,13 @@
 -------------------------------------------------------------------------
 -- @author Sébastien Gross &lt;seb•ɱɩɲʋʃ•awesome•ɑƬ•chezwam•ɖɵʈ•org&gt
 -- @copyright 2009 Sébastien Gross
--- @release v3.4.14
+-- @release v3.5.1
 -------------------------------------------------------------------------
 
 local mouse = mouse
-local wibox = wibox
 local screen = screen
 local timer = timer
+local wibox = require("wibox")
 local a_placement = require("awful.placement")
 local a_wibox = require("awful.wibox")
 local beautiful = require("beautiful")
@@ -31,7 +31,9 @@ local setmetatable = setmetatable
 -- The mouse_finder color<br/>
 -- <code>theme.mouse_finder_color = "#ff0000"</code><br/>
 -- </p>
-module("awful.mouse.finder")
+
+-- awful.mouse.finder
+local finder = { mt = {} }
 
 -- Mouse finder private data.
 -- @name data
@@ -77,8 +79,6 @@ local function show(self)
     -- do nothing if the mouse finder is already shown
     if data[self].wibox.visible then return end
     if not data[self].timer.started then
-        -- make sure the mouse finder is on the same screen as the mouse
-        data[self].wibox.screen = mouse.screen
         data[self].wibox:geometry({width = data[self].radius, height = data[self].radius })
         a_wibox.rounded_corners(data[self].wibox, data[self].radius/2 -1)
         data[self].timer:start()
@@ -114,7 +114,7 @@ end
 
 --- Find the mouse on the screen
 -- @param self A mouse finder object.
-function find(self)
+function finder.find(self)
     show(self)
 end
 
@@ -137,14 +137,18 @@ local function new()
     -- setup the timer action only if needed
     data[self].timer = timer { timeout = data[self].timeout }
     data[self].animate_timer = timer { timeout = data[self].animate_timeout }
-    data[self].timer:add_signal("timeout", data[self].hide)
-    data[self].animate_timer:add_signal("timeout", data[self].animate)
+    data[self].timer:connect_signal("timeout", data[self].hide)
+    data[self].animate_timer:connect_signal("timeout", data[self].animate)
     data[self].wibox.ontop = true
     data[self].wibox.visible = false
 
     return self
 end
 
-setmetatable(_M, { __call = function(_, ...) return new(...) end })
+function finder.mt:__call(...)
+    return new(...)
+end
+
+return setmetatable(finder, finder.mt)
 
 -- vim: ft=lua:et:sw=4:ts=4:sts=4:enc=utf-8:tw=78
