@@ -1,7 +1,7 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2008 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -17,9 +17,11 @@ local capi =
 }
 local client = require("awful.client")
 local layout = require("awful.layout")
+local a_screen = require("awful.screen")
 
 --- Places client according to special criteria.
-module("awful.placement")
+-- awful.placement
+local placement = {}
 
 -- Check if an area intersect another area.
 -- @param a The area.
@@ -102,20 +104,21 @@ end
 --- Place the client so no part of it will be outside the screen.
 -- @param c The client.
 -- @return The new client geometry.
-function no_offscreen(c)
+function placement.no_offscreen(c)
     local c = c or capi.client.focus
     local geometry = c:geometry()
-    local border = c.border_width
-    local screen_geometry = capi.screen[c.screen].workarea
+    local screen   = c.screen or a_screen.getbycoord(geometry.x, geometry.y)
+    local border   = c.border_width
+    local screen_geometry = capi.screen[screen].workarea
 
     if geometry.x + geometry.width + 2*border > screen_geometry.x + screen_geometry.width then
-        geometry.x = screen_geometry.x + screen_geometry.width - geometry.width
+        geometry.x = screen_geometry.x + screen_geometry.width - geometry.width - 2*border
     elseif geometry.x < screen_geometry.x then
         geometry.x = screen_geometry.x
     end
 
     if geometry.y + geometry.height + border > screen_geometry.y + screen_geometry.height then
-        geometry.y = screen_geometry.y + screen_geometry.height - geometry.height
+        geometry.y = screen_geometry.y + screen_geometry.height - geometry.height - 2*border
     elseif geometry.y < screen_geometry.y then
         geometry.y = screen_geometry.y
     end
@@ -125,11 +128,12 @@ end
 
 --- Place the client where there's place available with minimum overlap.
 -- @param c The client.
-function no_overlap(c)
-    local cls = client.visible(c.screen)
-    local curlay = layout.get()
-    local areas = { capi.screen[c.screen].workarea }
+function placement.no_overlap(c)
     local geometry = c:geometry()
+    local screen   = c.screen or a_screen.getbycoord(geometry.x, geometry.y)
+    local cls = client.visible(screen)
+    local curlay = layout.get()
+    local areas = { capi.screen[screen].workarea }
     for i, cl in pairs(cls) do
         if cl ~= c and cl.type ~= "desktop" and (client.floating.get(cl) or curlay == layout.suit.floating) then
             areas = area_remove(areas, cl:geometry())
@@ -177,7 +181,7 @@ end
 --- Place the client under the mouse.
 -- @param c The client.
 -- @return The new client geometry.
-function under_mouse(c)
+function placement.under_mouse(c)
     local c = c or capi.client.focus
     local c_geometry = c:geometry()
     local m_coords = capi.mouse.coords()
@@ -189,14 +193,15 @@ end
 -- @param c The client.
 -- @param p The parent (optional, nil for screen centering).
 -- @return The new client geometry.
-function centered(c, p)
+function placement.centered(c, p)
     local c = c or capi.client.focus
     local c_geometry = c:geometry()
+    local screen = c.screen or a_screen.getbycoord(c_geometry.x, c_geometry.y)
     local s_geometry
     if p then
         s_geometry = p:geometry()
     else
-        s_geometry = capi.screen[c.screen].geometry
+        s_geometry = capi.screen[screen].geometry
     end
     return c:geometry({ x = s_geometry.x + (s_geometry.width - c_geometry.width) / 2,
                         y = s_geometry.y + (s_geometry.height - c_geometry.height) / 2 })
@@ -206,14 +211,15 @@ end
 -- @param c The client.
 -- @param p The parent (optional, nil for screen centering).
 -- @return The new client geometry.
-function center_horizontal(c, p)
+function placement.center_horizontal(c, p)
     local c = c or capi.client.focus
     local c_geometry = c:geometry()
+    local screen = c.screen or a_screen.getbycoord(c_geometry.x, c_geometry.y)
     local s_geometry
     if p then
         s_geometry = p:geometry()
     else
-        s_geometry = capi.screen[c.screen].geometry
+        s_geometry = capi.screen[screen].geometry
     end
     return c:geometry({ x = s_geometry.x + (s_geometry.width - c_geometry.width) / 2 })
 end
@@ -222,17 +228,19 @@ end
 -- @param c The client.
 -- @param p The parent (optional, nil for screen centering).
 -- @return The new client geometry.
-function center_vertical(c, p)
+function placement.center_vertical(c, p)
     local c = c or capi.client.focus
     local c_geometry = c:geometry()
+    local screen = c.screen or a_screen.getbycoord(c_geometry.x, c_geometry.y)
     local s_geometry
     if p then
         s_geometry = p:geometry()
     else
-        s_geometry = capi.screen[c.screen].geometry
+        s_geometry = capi.screen[screen].geometry
     end
     return c:geometry({ y = s_geometry.y + (s_geometry.height - c_geometry.height) / 2 })
 end
 
+return placement
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
