@@ -1,7 +1,7 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2008 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -13,14 +13,15 @@ local ipairs = ipairs
 local pcall = pcall
 local capi =
 {
-    keygrabber = keygrabber,
     selection = selection
 }
+local keygrabber = require("awful.keygrabber")
 local util = require("awful.util")
 local beautiful = require("beautiful")
 
 --- Prompt module for awful
-module("awful.prompt")
+-- awful.prompt
+local prompt = {}
 
 --- Private data
 local data = {}
@@ -131,7 +132,7 @@ end
 local function prompt_text_with_cursor(args)
     local char, spacer, text_start, text_end, ret
     local text = args.text or ""
-    local prompt = args.prompt or ""
+    local _prompt = args.prompt or ""
     local underline = args.cursor_ul or "none"
 
     if args.selectall then
@@ -151,8 +152,7 @@ local function prompt_text_with_cursor(args)
         text_end = util.escape(text:sub(args.cursor_pos + 1))
     end
 
-    ret = prompt .. text_start .. "<span background=\"" .. util.color_strip_alpha(args.cursor_color) .. "\" foreground=\"" .. util.color_strip_alpha(args.text_color) .. "\" underline=\"" .. underline .. "\">" .. char .. "</span>" .. text_end .. spacer
-    if args.font then ret = "<span font_desc='" .. args.font .. "'>" .. ret .. "</span>" end
+    ret = _prompt .. text_start .. "<span background=\"" .. util.color_strip_alpha(args.cursor_color) .. "\" foreground=\"" .. util.color_strip_alpha(args.text_color) .. "\" underline=\"" .. underline .. "\">" .. char .. "</span>" .. text_end .. spacer
     return ret
 end
 
@@ -164,7 +164,10 @@ end
 -- @param history_path Optional parameter: file path where the history should be saved, set nil to disable history
 -- @param history_max Optional parameter: set the maximum entries in history file, 50 by default
 -- @param done_callback Optional parameter: the callback function to always call without arguments, regardless of whether the prompt was cancelled.
-function run(args, textbox, exe_callback, completion_callback, history_path, history_max, done_callback)
+-- @param changed_callback Optional parameter: the callback function to call with command as argument when a command was changed.
+-- @param keypressed_callback Optional parameter: the callback function to call with mod table, key and command as arguments when a key was pressed.
+function prompt.run(args, textbox, exe_callback, completion_callback, history_path, history_max, done_callback, changed_callback, keypressed_callback)
+    local grabber
     local theme = beautiful.get()
     if not args then args = {} end
     local command = args.text or ""
@@ -189,29 +192,65 @@ function run(args, textbox, exe_callback, completion_callback, history_path, his
     if not textbox or not exe_callback then
         return
     end
-    textbox.text = prompt_text_with_cursor{
+    textbox:set_font(font)
+    textbox:set_markup(prompt_text_with_cursor{
         text = text, text_color = inv_col, cursor_color = cur_col,
         cursor_pos = cur_pos, cursor_ul = cur_ul, selectall = selectall,
-        font = font, prompt = prettyprompt }
+        prompt = prettyprompt })
 
     local exec = function()
-        textbox.text = ""
+        textbox:set_markup("")
         history_add(history_path, command)
-        capi.keygrabber.stop()
+        keygrabber.stop(grabber)
         exe_callback(command)
         if done_callback then done_callback() end
     end
 
-    capi.keygrabber.run(
+    -- Update textbox
+    local function update()
+        textbox:set_font(font)
+        textbox:set_markup(prompt_text_with_cursor{
+                               text = command, text_color = inv_col, cursor_color = cur_col,
+                               cursor_pos = cur_pos, cursor_ul = cur_ul, selectall = selectall,
+                               prompt = prettyprompt })
+    end
+
+    grabber = keygrabber.run(
     function (modifiers, key, event)
-        if event ~= "press" then return true end
+        if event ~= "press" then return end
         -- Convert index array to hash table
         local mod = {}
         for k, v in ipairs(modifiers) do mod[v] = true end
+
+        -- Call the user specified callback. If it returns true as
+        -- the first result then return from the function. Treat the
+        -- second and third results as a new command and new prompt
+        -- to be set (if provided)
+        if keypressed_callback then
+            local user_catched, new_command, new_prompt =
+                keypressed_callback(mod, key, command)
+            if new_command or new_prompt then
+                if new_command then
+                    command = new_command
+                end
+                if new_prompt then
+                    prettyprompt = new_prompt
+                end
+                update()
+            end
+            if user_catched then
+                if changed_callback then
+                    changed_callback(command)
+                end
+                return
+            end
+        end
+
         -- Get out cases
         if (mod.Control and (key == "c" or key == "g"))
             or (not mod.Control and key == "Escape") then
-            textbox.text = ""
+            keygrabber.stop(grabber)
+            textbox:set_markup("")
             if done_callback then done_callback() end
             return false
         elseif (mod.Control and (key == "j" or key == "m"))
@@ -220,7 +259,7 @@ function run(args, textbox, exe_callback, completion_callback, history_path, his
             exec()
             -- We already unregistered ourselves so we don't want to return
             -- true, otherwise we may unregister someone else.
-            return true
+            return
         end
 
         -- Control cases
@@ -312,14 +351,15 @@ function run(args, textbox, exe_callback, completion_callback, history_path, his
             if completion_callback then
                 if key == "Tab" or key == "ISO_Left_Tab" then
                     if key == "ISO_Left_Tab" then
-                        if ncomp == 1 then return true end
+                        if ncomp == 1 then return end
                         if ncomp == 2 then
                             command = command_before_comp
-                            textbox.text = prompt_text_with_cursor{
+                            textbox:set_font(font)
+                            textbox:set_markup(prompt_text_with_cursor{
                                 text = command_before_comp, text_color = inv_col, cursor_color = cur_col,
                                 cursor_pos = cur_pos, cursor_ul = cur_ul, selectall = selectall,
-                                font = font, prompt = prettyprompt }
-                            return true
+                                prompt = prettyprompt })
+                            return
                         end
 
                         ncomp = ncomp - 2
@@ -334,7 +374,7 @@ function run(args, textbox, exe_callback, completion_callback, history_path, his
                     -- execute if only one match found and autoexec flag set
                     if matches and #matches == 1 and args.autoexec then
                         exec()
-                        return true
+                        return
                     end
                 else
                     ncomp = 1
@@ -404,14 +444,6 @@ function run(args, textbox, exe_callback, completion_callback, history_path, his
             selectall = nil
         end
 
-        -- Update textbox
-        local function update()
-            textbox.text = prompt_text_with_cursor{
-                text = command, text_color = inv_col, cursor_color = cur_col,
-                cursor_pos = cur_pos, cursor_ul = cur_ul, selectall = selectall,
-                font = font, prompt = prettyprompt }
-        end
-
         local success = pcall(update)
         while not success do
             -- TODO UGLY HACK TODO
@@ -428,8 +460,12 @@ function run(args, textbox, exe_callback, completion_callback, history_path, his
             success = pcall(update)
         end
 
-        return true
+        if changed_callback then
+            changed_callback(command)
+        end
     end)
 end
 
+return prompt
+
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
