@@ -1,29 +1,29 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
 require("awful.dbus")
-local loadstring = loadstring
+local load = loadstring or load -- v5.1 - loadstring, v5.2 - load
 local tostring = tostring
 local ipairs = ipairs
 local table = table
+local unpack = unpack or table.unpack -- v5.1: unpack, v5.2: table.unpack
 local dbus = dbus
-local unpack = unpack
 local type = type
 
 --- Remote control module allowing usage of awesome-client.
-module("awful.remote")
+-- awful.remote
 
 if dbus then
-    dbus.add_signal("org.naquadah.awesome.awful.Remote", function(data, code)
+    dbus.connect_signal("org.naquadah.awesome.awful.Remote", function(data, code)
         if data.member == "Eval" then
-            local f, e = loadstring(code)
+            local f, e = load(code)
             if f then
-                results = { f() }
-                retvals = {}
+                local results = { f() }
+                local retvals = {}
                 for _, v in ipairs(results) do
                     local t = type(v)
                     if t == "boolean" then
