@@ -1,7 +1,7 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -14,7 +14,8 @@ local aclient = require("awful.client")
 local atag = require("awful.tag")
 
 --- Apply rules to clients at startup.
-module("awful.rules")
+-- awful.rules
+local rules = {}
 
 --- This is the global rules table.
 -- <p>You should fill this table with your rule and properties to apply.
@@ -48,12 +49,15 @@ module("awful.rules")
 --   properties = { tag = mytagobject, switchtotag = true } }
 -- </code>
 -- </p>
--- <p>If you want to apply a custom callback to execute when a rule matched, you
+-- <p>If you want to apply a custom callback to execute when a rule matched,
+-- for example to pause playing music from mpd when you start dosbox, you
 -- can add:
 -- <br/>
 -- <code>
 -- { rule = { class = "dosbox" },
---   callback = awful.placement.centered }
+--   callback = function(c)
+--      awful.util.spawn('mpc pause')
+--   end }
 -- </code>
 -- </p>
 -- <p>Note that all "rule" entries need to match. If any of the entry does not
@@ -61,6 +65,8 @@ module("awful.rules")
 -- <p>If a client matches multiple rules, their applied in the order they are
 -- put in this global rules table. If the value of a rule is a string, then the
 -- match function is used to determine if the client matches the rule.</p>
+-- <p>If the value of a property is a function, that function gets called and
+-- function's return value is used for the property.</p>
 --
 -- <p> To match multiple clients to a rule one need to use slightly different 
 -- syntax:
@@ -97,13 +103,13 @@ module("awful.rules")
 --
 -- @class table
 -- @name rules
-rules = {}
+rules.rules = {}
 
 --- Check if a client match a rule.
 -- @param c The client.
 -- @param rule The rule to check.
 -- @return True if it matches, false otherwise.
-function match(c, rule)
+function rules.match(c, rule)
     if not rule then return false end
     for field, value in pairs(rule) do
         if c[field] then
@@ -123,9 +129,9 @@ end
 
 --- Check if a client match a rule. Multiple clients can be matched
 -- @param c The client.
--- @param rules The rule to check.
+-- @param rule The rule to check.
 -- @return True if at least one rule is matched, false otherwise.
-function match_any(c, rule)
+function rules.match_any(c, rule)
     if not rule then return false end
     for field, values in pairs(rule) do
         if c[field] then
@@ -143,12 +149,12 @@ end
 
 --- Apply rules to a client.
 -- @param c The client.
-function apply(c)
+function rules.apply(c)
     local props = {}
     local callbacks = {}
-    for _, entry in ipairs(rules) do
-        if (match(c, entry.rule) or match_any(c, entry.rule_any)) and
-            (not match(c, entry.except) and not match_any(c, entry.except_any)) then
+    for _, entry in ipairs(rules.rules) do
+        if (rules.match(c, entry.rule) or rules.match_any(c, entry.rule_any)) and
+            (not rules.match(c, entry.except) and not rules.match_any(c, entry.except_any)) then
             if entry.properties then
                 for property, value in pairs(entry.properties) do
                     props[property] = value
@@ -161,11 +167,14 @@ function apply(c)
     end
 
     for property, value in pairs(props) do
+        if property ~= "focus" and type(value) == "function" then
+            value = value(c)
+        end
         if property == "floating" then
             aclient.floating.set(c, value)
         elseif property == "tag" then
             c:tags({ value })
-            c.screen = value.screen
+            c.screen = atag.getscreen(value)
         elseif property == "switchtotag" and value and props.tag then
             atag.viewonly(props.tag)
         elseif property == "height" or property == "width" or
@@ -173,6 +182,8 @@ function apply(c)
             local geo = c:geometry();
             geo[property] = value
             c:geometry(geo);
+        elseif property == "focus" then
+            -- This will be handled below
         elseif type(c[property]) == "function" then
             c[property](c, value)
         else
@@ -192,12 +203,14 @@ function apply(c)
 
     -- Do this at last so we do not erase things done by the focus
     -- signal.
-    if props.focus then
+    if props.focus and (type(props.focus) ~= "function" or props.focus(c)) then
         client.focus = c
     end
 end
 
-client.add_signal("manage", apply)
-client.remove_signal("manage", atag.withcurrent)
+client.connect_signal("manage", rules.apply)
+client.disconnect_signal("manage", atag.withcurrent)
+
+return rules
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
