@@ -1,7 +1,7 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -14,11 +14,11 @@ local capi =
 }
 
 --- Startup notification module for awful
-module("awful.startup_notification")
+-- awful.startup_notification
 
 local app_starting = {}
 
-cursor_waiting = "watch"
+local cursor_waiting = "watch"
 
 local function update_cursor()
     if #app_starting > 0 then
@@ -46,9 +46,9 @@ end
 local function unregister_hook(event) unregister_event(event.id) end
 local function register_hook(event) register_event(event.id) end
 
-capi.awesome.add_signal("spawn::initiated", register_hook)
-capi.awesome.add_signal("spawn::canceled", unregister_hook)
-capi.awesome.add_signal("spawn::completed", unregister_hook)
-capi.awesome.add_signal("spawn::timeout", unregister_hook)
+capi.awesome.connect_signal("spawn::initiated", register_hook)
+capi.awesome.connect_signal("spawn::canceled", unregister_hook)
+capi.awesome.connect_signal("spawn::completed", unregister_hook)
+capi.awesome.connect_signal("spawn::timeout", unregister_hook)
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
