@@ -1,16 +1,16 @@
 -------------------------------------------------------------------------
 -- @author Sébastien Gross &lt;seb•ɱɩɲʋʃ•awesome•ɑƬ•chezwam•ɖɵʈ•org&gt
 -- @copyright 2009 Sébastien Gross
--- @release v3.4.14
+-- @release v3.5.1
 -------------------------------------------------------------------------
 
 local mouse = mouse
-local widget = widget
-local wibox = wibox
 local screen = screen
 local timer = timer
+local wibox = require("wibox")
 local a_placement = require("awful.placement")
 local beautiful = require("beautiful")
+local textbox = require("wibox.widget.textbox")
 local setmetatable = setmetatable
 local ipairs = ipairs
 
@@ -18,7 +18,7 @@ local ipairs = ipairs
 -- A tooltip is a small hint displayed when the mouse cursor
 -- hovers a specific item.
 -- In awesome, a tooltip can be linked with almost any
--- object having a <code>add_signal()</code> method and receiving
+-- object having a <code>connect_signal()</code> method and receiving
 -- <code>mouse::enter</code> and <code>mouse::leave</code> signals.
 -- <p>How to create a tooltip?<br/>
 --  <code>
@@ -46,7 +46,8 @@ local ipairs = ipairs
 -- </code>
 -- Now the same tooltip is only attached to <code>myclock</code>.<br/>
 -- </p>
-module("awful.tooltip")
+-- awful.tooltip
+local tooltip = { mt = {} }
 
 local data = setmetatable({}, { __mode = 'k' })
 
@@ -77,13 +78,9 @@ end
 local function set_geometry(self)
     local my_geo = self.wibox:geometry()
     -- calculate width / height
-    n_s = self.wibox.widgets[1]:extents()
-    if my_geo.width ~= n_s.width or my_geo.height ~= n_s.height then
-        self.wibox:geometry(n_s)
-        place(self)
-    end
-    if not self.wibox.visible then
-       place(self)
+    local n_w, n_h = self.widget:fit(-1, -1)
+    if my_geo.width ~= n_w or my_geo.height ~= n_h then
+        self.wibox:geometry({ width = n_w, height = n_h })
     end
 end
 
@@ -92,8 +89,6 @@ end
 local function show(self)
     -- do nothing if the tooltip is already shown
     if self.visible then return end
-    -- make sure the tooltip is on the same screen as the mouse
-    self.wibox.screen = mouse.screen
     if data[self].timer then
         if not data[self].timer.started then
             data[self].timer_function()
@@ -101,6 +96,7 @@ local function show(self)
         end
     end
     set_geometry(self)
+    place(self)
     self.wibox.visible = true
     self.visible = true
 end
@@ -123,8 +119,10 @@ end
 -- @param self The tooltip object.
 -- @param text New tooltip text.
 local function set_text(self, text)
-    self.wibox.widgets[1].text = '<span color="' .. data[self].fg
-        .. '" font_desc="' .. data[self].font .. '">' .. text .. "</span>"
+    self.widget:set_font(data[self].font)
+    self.widget:set_markup('<span color="' .. data[self].fg
+        .. '">' .. text .. "</span>")
+    set_geometry(self)
 end
 
 --- Change the tooltip's update interval.
@@ -142,7 +140,7 @@ local function set_defaults(self)
     self.wibox.border_width = beautiful.tooltip_border_width or beautiful.border_width or 1
     self.wibox.border_color = beautiful.tooltip_border_color or beautiful.border_normal or "#ffcb60"
     self.wibox.opacity = beautiful.tooltip_opacity or 1
-    self.wibox.bg = beautiful.tooltip_bg_color or beautiful.bg_focus or "#ffcb60"
+    self.wibox:set_bg(beautiful.tooltip_bg_color or beautiful.bg_focus or "#ffcb60")
     data[self].fg = beautiful.tooltip_fg_color or beautiful.fg_focus or "#000000"
     data[self].font = beautiful.tooltip_font or beautiful.font or "terminus 6"
 end
@@ -151,23 +149,23 @@ end
 -- @param self The tooltip.
 -- @param object An object.
 local function add_to_object(self, object)
-    object:add_signal("mouse::enter", data[self].show)
-    object:add_signal("mouse::leave", data[self].hide)
+    object:connect_signal("mouse::enter", data[self].show)
+    object:connect_signal("mouse::leave", data[self].hide)
 end
 
 --- Remove tooltip from an object.
 -- @param self The tooltip.
 -- @param object An object.
 local function remove_from_object(self, object)
-    object:remove_signal("mouse::enter", data[self].show)
-    object:remove_signal("mouse::leave", data[self].hide)
+    object:disconnect_signal("mouse::enter", data[self].show)
+    object:disconnect_signal("mouse::leave", data[self].hide)
 end
 
 
 --- Create a new tooltip and link it to a widget.
 -- @param args Arguments for tooltip creation may containt:<br/>
 -- <code>timeout</code>: The timeout value for update_func.<br/>
--- <code>timer_function</code>: A function to dynamicaly change the tooltip
+-- <code>timer_function</code>: A function to dynamically change the tooltip
 --     text.<br/>
 -- <code>objects</code>: A list of objects linked to the tooltip.<br/>
 -- @return The created tooltip.
@@ -180,10 +178,7 @@ local function new(args)
         visible = false,
     }
 
-    local my_textbox = widget({
-        type = "textbox",
-        name = "tooltip_textbox",
-        align="right"})
+    local my_textbox = textbox()
 
     -- private data
     data[self] = {
@@ -204,19 +199,19 @@ local function new(args)
         data[self].timer = timer { timeout = args.timeout and args.timeout or 1 }
         data[self].timer_function = function()
                 self:set_text(args.timer_function())
-                set_geometry(self)
             end
-        data[self].timer:add_signal("timeout", data[self].timer_function)
+        data[self].timer:connect_signal("timeout", data[self].timer_function)
     end
 
     -- set tooltip properties
     self.wibox.visible = false
-    -- Who want a non ontop tooltip ?
+    -- Who wants a non ontop tooltip ?
     self.wibox.ontop = true
-    self.wibox.widgets = { my_textbox }
+    self.wibox:set_widget(my_textbox)
+    self.widget = my_textbox
 
     -- add some signals on both the tooltip and widget
-    self.wibox:add_signal("mouse::enter", data[self].hide)
+    self.wibox:connect_signal("mouse::enter", data[self].hide)
 
     -- Add tooltip to objects
     if args.objects then
@@ -228,6 +223,10 @@ local function new(args)
     return self
 end
 
-setmetatable(_M, { __call = function(_, ...) return new(...) end })
+function tooltip.mt:__call(...)
+    return new(...)
+end
+
+return setmetatable(tooltip, tooltip.mt)
 
 -- vim: ft=lua:et:sw=4:ts=4:sts=4:enc=utf-8:tw=78
