@@ -1,14 +1,14 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2008 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
 local os = os
 local io = io
 local assert = assert
-local loadstring = loadstring
+local load = loadstring or load -- v5.1 - loadstring, v5.2 - load
 local loadfile = loadfile
 local debug = debug
 local pairs = pairs
@@ -24,13 +24,13 @@ local capi =
 }
 
 --- Utility module for awful
-module("awful.util")
+-- awful.util
+local util = {}
+util.table = {}
 
-table = {}
+util.shell = os.getenv("SHELL") or "/bin/sh"
 
-shell = os.getenv("SHELL") or "/bin/sh"
-
-function deprecate(see)
+function util.deprecate(see)
     io.stderr:write("W: awful: function is deprecated")
     if see then
         io.stderr:write(", see " .. see)
@@ -42,7 +42,7 @@ end
 --- Strip alpha part of color.
 -- @param color The color.
 -- @return The color without alpha channel.
-function color_strip_alpha(color)
+function util.color_strip_alpha(color)
     if color:len() == 9 then
         color = color:sub(1, 7)
     end
@@ -53,17 +53,19 @@ end
 -- @param t A length. Must be greater than zero.
 -- @param i An absolute index to fit into #t.
 -- @return An integer in (1, t) or nil if t is less than or equal to zero.
-function cycle(t, i)
+function util.cycle(t, i)
     if t < 1 then return end
-    while i > t do i = i - t end
-    while i < 1 do i = i + t end
+    i = i % t
+    if i == 0 then
+        i = t
+    end
     return i
 end
 
 --- Create a directory
 -- @param dir The directory.
 -- @return mkdir return code
-function mkdir(dir)
+function util.mkdir(dir)
     return os.execute("mkdir -p " .. dir)
 end
 
@@ -72,7 +74,7 @@ end
 -- @param sn Enable startup-notification.
 -- @param screen The screen where to spawn window.
 -- @return The awesome.spawn return value.
-function spawn(cmd, sn, screen)
+function util.spawn(cmd, sn, screen)
     if cmd and cmd ~= "" then
         if sn == nil then sn = true end
         return capi.awesome.spawn(cmd, sn, screen or capi.mouse.screen)
@@ -82,9 +84,9 @@ end
 --- Spawn a program using the shell.
 -- @param cmd The command.
 -- @param screen The screen where to run the command.
-function spawn_with_shell(cmd, screen)
+function util.spawn_with_shell(cmd, screen)
     if cmd and cmd ~= "" then
-        cmd = shell .. " -c \"" .. cmd .. "\""
+        cmd = { util.shell, "-c", cmd }
         return capi.awesome.spawn(cmd, false, screen or capi.mouse.screen)
     end
 end
@@ -92,7 +94,7 @@ end
 --- Read a program output and returns its output as a string.
 -- @param cmd The command to run.
 -- @return A string with the program output, or the error if one occured.
-function pread(cmd)
+function util.pread(cmd)
     if cmd and cmd ~= "" then
         local f, err = io.popen(cmd, 'r')
         if f then
@@ -107,8 +109,8 @@ end
 
 --- Eval Lua code.
 -- @return The return value of Lua code.
-function eval(s)
-    return assert(loadstring(s))()
+function util.eval(s)
+    return assert(load(s))()
 end
 
 local xml_entity_names = { ["'"] = "&apos;", ["\""] = "&quot;", ["<"] = "&lt;", [">"] = "&gt;", ["&"] = "&amp;" };
@@ -116,7 +118,7 @@ local xml_entity_names = { ["'"] = "&apos;", ["\""] = "&quot;", ["<"] = "&lt;",
 -- Useful to set raw text in textbox.
 -- @param text Text to escape.
 -- @return Escape text.
-function escape(text)
+function util.escape(text)
     return text and text:gsub("['&<>\"]", xml_entity_names) or nil
 end
 
@@ -124,7 +126,7 @@ local xml_entity_chars = { lt = "<", gt = ">", nbsp = " ", quot = "\"", apos = "
 --- Unescape a string from entities.
 -- @param text Text to unescape.
 -- @return Unescaped text.
-function unescape(text)
+function util.unescape(text)
     return text and text:gsub("&(%a+);", xml_entity_chars) or nil
 end
 
@@ -133,7 +135,7 @@ end
 -- @param path The file path.
 -- @return A function if everything is alright, a string with the error
 -- otherwise.
-function checkfile(path)
+function util.checkfile(path)
     local f, e = loadfile(path)
     -- Return function if function, otherwise return error.
     if f then return f end
@@ -144,8 +146,8 @@ end
 -- It checks if the configuration file is valid, and then restart if it's ok.
 -- If it's not ok, the error will be returned.
 -- @return Never return if awesome restart, or return a string error.
-function restart()
-    local c = checkfile(capi.awesome.conffile)
+function util.restart()
+    local c = util.checkfile(capi.awesome.conffile)
 
     if type(c) ~= "function" then
         return c
@@ -159,7 +161,7 @@ end
 -- default paths.
 -- @param d The directory to get (either "config" or "cache").
 -- @return A string containing the requested path.
-function getdir(d)
+function util.getdir(d)
     if d == "config" then
         local dir = os.getenv("XDG_CONFIG_HOME")
         if dir then
@@ -180,23 +182,33 @@ end
 -- @param iconname The name of the icon to search for.
 -- @param exts Table of image extensions allowed, otherwise { 'png', gif' }
 -- @param dirs Table of dirs to search, otherwise { '/usr/local/share/pixmaps/' }
-function geticonpath(iconname, exts, dirs)
+-- @param size Optional size. If this is specified, subdirectories <size>x<size>
+--             of the dirs are searched first
+function util.geticonpath(iconname, exts, dirs, size)
     exts = exts or { 'png', 'gif' }
     dirs = dirs or { '/usr/local/share/pixmaps/' }
     for _, d in pairs(dirs) do
         for _, e in pairs(exts) do
-            local icon = d .. iconname .. '.' .. e
-            if file_readable(icon) then
+            local icon
+            if size then
+                icon = string.format("%s%ux%u/%s.%s",
+                       d, size, size, iconname, e)
+                if util.file_readable(icon) then
+                    return icon
+                end
+            end
+            icon = d .. iconname .. '.' .. e
+            if util.file_readable(icon) then
                 return icon
-            end 
-        end 
-    end 
+            end
+        end
+    end
 end
 
 --- Check if file exists and is readable.
 -- @param filename The file path
 -- @return True if file exists and readable.
-function file_readable(filename)
+function util.file_readable(filename)
     local file = io.open(filename)
     if file then
         io.close(file)
@@ -236,7 +248,7 @@ end
 -- { }, { 10 }, { 15 }, { 34 }, { 10, 15 }, { 10, 34 }, etc.
 -- @param set A set.
 -- @return A table with all subset.
-function subsets(set)
+function util.subsets(set)
     local mask = {}
     local ret = {}
     for i = 1, #set do mask[i] = false end
@@ -250,13 +262,85 @@ function subsets(set)
     return ret
 end
 
+-- Return true whether rectangle B is in the right direction
+-- compared to rectangle A.
+-- @param dir The direction.
+-- @param gA The geometric specification for rectangle A.
+-- @param gB The geometric specification for rectangle B.
+-- @return True if B is in the direction of A.
+local function is_in_direction(dir, gA, gB)
+    if dir == "up" then
+        return gA.y > gB.y
+    elseif dir == "down" then
+        return gA.y < gB.y
+    elseif dir == "left" then
+        return gA.x > gB.x
+    elseif dir == "right" then
+        return gA.x < gB.x
+    end
+    return false
+end
+
+-- Calculate distance between two points.
+-- i.e: if we want to move to the right, we will take the right border
+-- of the currently focused screen and the left side of the checked screen.
+-- @param dir The direction.
+-- @param gA The first rectangle.
+-- @param gB The second rectangle.
+-- @return The distance between the screens.
+local function calculate_distance(dir, _gA, _gB)
+    local gAx = _gA.x
+    local gAy = _gA.y
+    local gBx = _gB.x
+    local gBy = _gB.y
+
+    if dir == "up" then
+        gBy = _gB.y + _gB.height
+    elseif dir == "down" then
+        gAy = _gA.y + _gA.height
+    elseif dir == "left" then
+        gBx = _gB.x + _gB.width
+    elseif dir == "right" then
+        gAx = _gA.x + _gA.width
+    end
+
+    return math.sqrt(math.pow(gBx - gAx, 2) + math.pow(gBy - gAy, 2))
+end
+
+-- Get the nearest rectangle in the given direction. Every rectangle is specified as a table
+-- with 'x', 'y', 'width', 'height' keys, the same as client or screen geometries.
+-- @param dir The direction, can be either "up", "down", "left" or "right".
+-- @param recttbl A table of rectangle specifications.
+-- @param cur The current rectangle.
+-- @return The index for the rectangle in recttbl closer to cur in the given direction. nil if none found.
+function util.get_rectangle_in_direction(dir, recttbl, cur)
+    local dist, dist_min
+    local target = nil
+
+    -- We check each object
+    for i, rect in ipairs(recttbl) do
+        -- Check geometry to see if object is located in the right direction.
+        if is_in_direction(dir, cur, rect) then
+            -- Calculate distance between current and checked object.
+            dist = calculate_distance(dir, cur, rect)
+
+            -- If distance is shorter then keep the object.
+            if not target or dist < dist_min then
+                target = i
+                dist_min = dist
+            end
+        end
+    end
+    return target
+end
+
 --- Join all tables given as parameters.
 -- This will iterate all tables and insert all their keys into a new table.
 -- @param args A list of tables to join
 -- @return A new table containing all keys from the arguments.
-function table.join(...)
+function util.table.join(...)
     local ret = {}
-    for i, t in ipairs({...}) do
+    for i, t in pairs({...}) do
         if t then
             for k, v in pairs(t) do
                 if type(k) == "number" then
@@ -274,7 +358,7 @@ end
 -- @param t The table.
 -- @param item The item to look for in values of the table.
 -- @return The key were the item is found, or nil if not found.
-function table.hasitem(t, item)
+function util.table.hasitem(t, item)
     for k, v in pairs(t) do
         if v == item then
             return k
@@ -287,7 +371,7 @@ end
 -- @param width Maximum length of each line. Default: 72.
 -- @param indent Number of spaces added before each wrapped line. Default: 0.
 -- @return The string with lines wrapped to width.
-function linewrap(text, width, indent)
+function util.linewrap(text, width, indent)
     local text = text or ""
     local width = width or 72
     local indent = indent or 0
@@ -305,7 +389,7 @@ end
 --- Get a sorted table with all integer keys from a table
 -- @param t the table for which the keys to get
 -- @return A table with keys
-function table.keys(t)
+function util.table.keys(t)
     local keys = { }
     for k, _ in pairs(t) do
         rtable.insert(keys, k)
@@ -320,8 +404,8 @@ end
 -- @param t The table to retrieve the keys for
 -- @param ... the types to look for
 -- @return A filtered table with keys
-function table.keys_filter(t, ...)
-    local keys = table.keys(t)
+function util.table.keys_filter(t, ...)
+    local keys = util.table.keys(t)
     local keys_filtered = { }
     for _, k in pairs(keys) do
         for _, et in pairs({...}) do
@@ -337,7 +421,7 @@ end
 --- Reverse a table
 -- @param t the table to reverse
 -- @return the reversed table
-function table.reverse(t)
+function util.table.reverse(t)
     local tr = { }
     -- reverse all elements with integer keys
     for _, v in ipairs(t) do
@@ -355,10 +439,14 @@ end
 --- Clone a table
 -- @param t the table to clone
 -- @return a clone of t
-function table.clone(t)
+function util.table.clone(t)
     local c = { }
     for k, v in pairs(t) do
-        c[k] = v
+        if type(v) == "table" then
+            c[k] = util.table.clone(v)
+        else
+            c[k] = v
+        end
     end
     return c
 end
@@ -366,11 +454,12 @@ end
 ---
 -- Returns an iterator to cycle through, starting from the first element or the
 -- given index, all elments of a table that match a given criteria.
+--
 -- @param t      the table to iterate
 -- @param filter a function that returns true to indicate a positive match
 -- @param start  what index to start iterating from.  Default is 1 (=> start of
 -- the table)
-function table.cycle(t, filter, start)
+function util.table.iterate(t, filter, start)
     local count  = 0
     local index  = start or 1
     local length = #t
@@ -378,10 +467,13 @@ function table.cycle(t, filter, start)
     return function ()
         while count < length do
             local item = t[index]
-            index = cycle(#t, index + 1)
+            index = util.cycle(#t, index + 1)
             count = count + 1
             if filter(item) then return item end
         end
     end
 end
+
+return util
+
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
