@@ -1,7 +1,7 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 -- Grab environment we need
@@ -9,21 +9,21 @@ local capi =
 {
     awesome = awesome,
     screen = screen,
-    wibox = wibox,
     client = client
 }
 local setmetatable = setmetatable
 local tostring = tostring
 local ipairs = ipairs
 local table = table
-local type = type
-local image = image
 local error = error
+local wibox = require("wibox")
+local beautiful = require("beautiful")
 
 --- Wibox module for awful.
 -- This module allows you to easily create wibox and attach them to the edge of
 -- a screen.
-module("awful.wibox")
+-- awful.wibox
+local awfulwibox = { mt = {} }
 
 -- Array of table with wiboxes inside.
 -- It's an array so it is ordered.
@@ -32,7 +32,7 @@ local wiboxes = {}
 --- Get a wibox position if it has been set, or return top.
 -- @param wibox The wibox
 -- @return The wibox position.
-function get_position(wibox)
+function awfulwibox.get_position(wibox)
     for _, wprop in ipairs(wiboxes) do
         if wprop.wibox == wibox then
             return wprop.position
@@ -46,8 +46,7 @@ end
 -- @param position The position: top, bottom left or right.
 -- @param screen If the wibox it not attached to a screen, specified on which
 -- screen the position should be set.
-function set_position(wibox, position, screen)
-    local screen = screen or wibox.screen or 1
+function awfulwibox.set_position(wibox, position, screen)
     local area = capi.screen[screen].geometry
 
     -- The "length" of a wibox is always chosen to be the optimal size
@@ -74,7 +73,7 @@ end
 -- Reset all wiboxes positions.
 local function update_all_wiboxes_position()
     for _, wprop in ipairs(wiboxes) do
-        set_position(wprop.wibox, wprop.position)
+        awfulwibox.set_position(wprop.wibox, wprop.position, wprop.screen)
     end
 end
 
@@ -106,7 +105,8 @@ end
 -- will be attached.
 -- @param wibox The wibox to attach.
 -- @param position The position of the wibox: top, bottom, left or right.
-function attach(wibox, position)
+-- @param screen TODO, this seems to be unused
+function awfulwibox.attach(wibox, position, screen)
     -- Store wibox as attached in a weak-valued table
     local wibox_prop_table
     -- Start from end since we sometimes remove items
@@ -123,20 +123,19 @@ function attach(wibox, position)
     end
 
     if not wibox_prop_table then
-        table.insert(wiboxes, setmetatable({ wibox = wibox, position = position }, { __mode = 'v' }))
+        table.insert(wiboxes, setmetatable({ wibox = wibox, position = position, screen = screen }, { __mode = 'v' }))
     else
         wibox_prop_table.position = position
     end
 
-    wibox:add_signal("property::width", wibox_update_strut)
-    wibox:add_signal("property::height", wibox_update_strut)
-    wibox:add_signal("property::visible", wibox_update_strut)
+    wibox:connect_signal("property::width", wibox_update_strut)
+    wibox:connect_signal("property::height", wibox_update_strut)
+    wibox:connect_signal("property::visible", wibox_update_strut)
 
-    wibox:add_signal("property::screen", call_wibox_position_hook_on_prop_update)
-    wibox:add_signal("property::width", call_wibox_position_hook_on_prop_update)
-    wibox:add_signal("property::height", call_wibox_position_hook_on_prop_update)
-    wibox:add_signal("property::visible", call_wibox_position_hook_on_prop_update)
-    wibox:add_signal("property::border_width", call_wibox_position_hook_on_prop_update)
+    wibox:connect_signal("property::width", call_wibox_position_hook_on_prop_update)
+    wibox:connect_signal("property::height", call_wibox_position_hook_on_prop_update)
+    wibox:connect_signal("property::visible", call_wibox_position_hook_on_prop_update)
+    wibox:connect_signal("property::border_width", call_wibox_position_hook_on_prop_update)
 end
 
 --- Align a wibox.
@@ -144,9 +143,8 @@ end
 -- @param align The alignment: left, right or center.
 -- @param screen If the wibox is not attached to any screen, you can specify the
 -- screen where to align. Otherwise 1 is assumed.
-function align(wibox, align, screen)
-    local position = get_position(wibox)
-    local screen = screen or wibox.screen or 1
+function awfulwibox.align(wibox, align, screen)
+    local position = awfulwibox.get_position(wibox)
     local area = capi.screen[screen].workarea
 
     if position == "right" then
@@ -190,10 +188,9 @@ end
 --- Stretch a wibox so it takes all screen width or height.
 -- @param wibox The wibox.
 -- @param screen The screen to stretch on, or the wibox screen.
-function stretch(wibox, screen)
-    local screen = screen or wibox.screen
+function awfulwibox.stretch(wibox, screen)
     if screen then
-        local position = get_position(wibox)
+        local position = awfulwibox.get_position(wibox)
         local area = capi.screen[screen].workarea
         if position == "right" or position == "left" then
             wibox.height = area.height - (2 * wibox.border_width)
@@ -206,19 +203,20 @@ function stretch(wibox, screen)
 end
 
 --- Create a new wibox and attach it to a screen edge.
--- @see capi.wibox
+-- @see wibox
 -- @param args A table with standard arguments to wibox() creator.
 -- You can add also position key with value top, bottom, left or right.
 -- You can also use width or height in % and set align to center, right or left.
 -- You can also set the screen key with a screen number to attach the wibox.
 -- If not specified, 1 is assumed.
 -- @return The wibox created.
-function new(arg)
+function awfulwibox.new(arg)
     local arg = arg or {}
     local position = arg.position or "top"
     local has_to_stretch = true
-    -- Empty position and align in arg so we are passing deprecation warning
-    arg.position = nil
+    local screen = arg.screen or 1
+
+    arg.type = arg.type or "dock"
 
     if position ~= "top" and position ~="bottom"
             and position ~= "left" and position ~= "right" then
@@ -228,7 +226,7 @@ function new(arg)
 
     -- Set default size
     if position == "left" or position == "right" then
-        arg.width = arg.width or capi.awesome.font_height * 1.5
+        arg.width = arg.width or beautiful.get_font_height(arg.font) * 1.5
         if arg.height then
             has_to_stretch = false
             if arg.screen then
@@ -239,7 +237,7 @@ function new(arg)
             end
         end
     else
-        arg.height = arg.height or capi.awesome.font_height * 1.5
+        arg.height = arg.height or beautiful.get_font_height(arg.font) * 1.5
         if arg.width then
             has_to_stretch = false
             if arg.screen then
@@ -251,80 +249,22 @@ function new(arg)
         end
     end
 
-    local w = capi.wibox(arg)
+    local w = wibox(arg)
 
-    if position == "left" then
-        w.orientation = "north"
-    elseif position == "right" then
-        w.orientation = "south"
-    end
+    w.visible = true
 
-    w.screen = arg.screen or 1
-
-    attach(w, position)
+    awfulwibox.attach(w, position, screen)
     if has_to_stretch then
-        stretch(w)
+        awfulwibox.stretch(w, screen)
     else
-        align(w, arg.align)
+        awfulwibox.align(w, arg.align, screen)
     end
 
-    set_position(w, position)
+    awfulwibox.set_position(w, position, screen)
 
     return w
 end
 
-local function do_rounded_corners(width, height, corner)
-    local img = image.argb32(width, height, nil)
-
-    -- The image starts completely black which is fully opaque for our use
-
-    local function transp_rect(x, y)
-        img:draw_rectangle(x, y, corner, corner, true, "#ffffff")
-    end
-    local function opaque_circle(x, y)
-        -- x, y are the center of the circle
-        img:draw_circle(x, y, corner, corner, true, "#000000")
-    end
-
-    -- Upper left corner
-    -- First make a 'corner times corner' rectangle transparent
-    transp_rect(0, 0)
-    -- Then add the rounded corner
-    opaque_circle(corner, corner)
-
-    -- Upper right corner
-    transp_rect(width - corner, 0)
-    opaque_circle(width - corner - 1, corner)
-
-    -- Bottom left corner
-    transp_rect(0, height - corner)
-    opaque_circle(corner, height - corner - 1)
-
-    -- Bottom right corner
-    transp_rect(width - corner, height - corner)
-    opaque_circle(width - corner - 1, height - corner - 1)
-
-    return img
-end
-
---- Add rounded corners to a wibox
--- @param wibox The wibox.
--- @param corner_size The size in pixel of the rounded corners.
-function rounded_corners(wibox, corner_size)
-    local border = wibox.border_width
-
-    -- Corners can't be larger than half the wibox' space
-    if wibox.width / 2 < corner_size then
-        corner_size = wibox.width / 2
-    end
-    if wibox.height / 2 < corner_size then
-        corner_size = wibox.height / 2
-    end
-
-    wibox.shape_clip = do_rounded_corners(wibox.width, wibox.height, corner_size)
-    wibox.shape_bounding = do_rounded_corners(wibox.width + border * 2, wibox.height + border * 2, corner_size + border)
-end
-
 local function update_wiboxes_on_struts(c)
     local struts = c:struts()
     if struts.left ~= 0 or struts.right ~= 0
@@ -334,12 +274,13 @@ local function update_wiboxes_on_struts(c)
 end
 
 -- Hook registered to reset all wiboxes position.
-capi.client.add_signal("manage", function(c)
-    update_wiboxes_on_struts(c)
-    c:add_signal("property::struts", update_wiboxes_on_struts)
-end)
-capi.client.add_signal("unmanage", update_wiboxes_on_struts)
+capi.client.connect_signal("property::struts", update_wiboxes_on_struts)
+capi.client.connect_signal("unmanage", update_wiboxes_on_struts)
 
-setmetatable(_M, { __call = function(_, ...) return new(...) end })
+function awfulwibox.mt:__call(...)
+    return awfulwibox.new(...)
+end
+
+return setmetatable(awfulwibox, awfulwibox.mt)
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
