@@ -1,7 +1,7 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 local setmetatable = setmetatable
@@ -10,44 +10,40 @@ local button = require("awful.button")
 local layout = require("awful.layout")
 local tag = require("awful.tag")
 local beautiful = require("beautiful")
-local capi = { image = image,
-               screen = screen,
-               widget = widget }
+local imagebox = require("wibox.widget.imagebox")
 
 --- Layoutbox widget.
-module("awful.widget.layoutbox")
+-- awful.widget.layoutbox
+local layoutbox = { mt = {} }
 
 local function update(w, screen)
     local layout = layout.getname(layout.get(screen))
-    if layout and beautiful["layout_" ..layout] then
-        w.image = capi.image(beautiful["layout_" ..layout])
-    else
-        w.image = nil
-    end
+    w:set_image(layout and beautiful["layout_" .. layout])
 end
 
 --- Create a layoutbox widget. It draws a picture with the current layout
 -- symbol of the current tag.
 -- @param screen The screen number that the layout will be represented for.
--- @param args Standard arguments for an imagebox widget.
 -- @return An imagebox widget configured as a layoutbox.
-function new(screen, args)
+function layoutbox.new(screen)
     local screen = screen or 1
-    local args = args or {}
-    args.type = "imagebox"
-    local w = capi.widget(args)
+    local w = imagebox()
     update(w, screen)
 
-    local function update_on_tag_selection(tag)
-        return update(w, tag.screen)
+    local function update_on_tag_selection(t)
+        return update(w, tag.getscreen(t))
     end
 
-    tag.attached_add_signal(screen, "property::selected", update_on_tag_selection)
-    tag.attached_add_signal(screen, "property::layout", update_on_tag_selection)
+    tag.attached_connect_signal(screen, "property::selected", update_on_tag_selection)
+    tag.attached_connect_signal(screen, "property::layout", update_on_tag_selection)
 
     return w
 end
 
-setmetatable(_M, { __call = function(_, ...) return new(...) end })
+function layoutbox.mt:__call(...)
+    return layoutbox.new(...)
+end
+
+return setmetatable(layoutbox, layoutbox.mt)
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
