@@ -1,18 +1,18 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 local setmetatable = setmetatable
 local ipairs = ipairs
 local math = math
-local capi = { image = image,
-               widget = widget }
-local layout = require("awful.widget.layout")
+local base = require("wibox.widget.base")
+local color = require("gears.color")
 
 --- A progressbar widget.
-module("awful.widget.progressbar")
+-- awful.widget.progressbar
+local progressbar = { mt = {} }
 
 local data = setmetatable({}, { __mode = "k" })
 
@@ -23,14 +23,6 @@ local data = setmetatable({}, { __mode = "k" })
 -- @param progressbar The progressbar.
 -- @param color The border color to set.
 
---- Set the progressbar foreground color as a gradient.
--- @name set_gradient_colors
--- @class function
--- @param progressbar The progressbar.
--- @param gradient_colors A table with gradients colors. The distance between each color
--- can also be specified. Example: { "red", "blue" } or { "red", "green",
--- "blue", blue = 10 } to specify blue distance from other colors.
-
 --- Set the progressbar foreground color.
 -- @name set_color
 -- @class function
@@ -74,18 +66,16 @@ local data = setmetatable({}, { __mode = "k" })
 -- @param value The value.
 
 local properties = { "width", "height", "border_color",
-                     "gradient_colors", "color", "background_color",
+                     "color", "background_color",
                      "vertical", "value", "max_value",
                      "ticks", "ticks_gap", "ticks_size" }
 
-local function update(pbar)
-    local width = data[pbar].width or 100
-    local height = data[pbar].height or 20
+function progressbar.draw(pbar, wibox, cr, width, height)
     local ticks_gap = data[pbar].ticks_gap or 1
     local ticks_size = data[pbar].ticks_size or 4
 
-    -- Create new empty image
-    local img = capi.image.argb32(width, height, nil)
+    -- We want one pixel wide lines
+    cr:set_line_width(1)
 
     local value = data[pbar].value
     local max_value = data[pbar].max_value
@@ -98,36 +88,29 @@ local function update(pbar)
     local border_width = 0
     if data[pbar].border_color then
         -- Draw border
-        img:draw_rectangle(0, 0, width, height, false, data[pbar].border_color)
+        cr:rectangle(0.5, 0.5, width - 1, height - 1)
+        cr:set_source(color(data[pbar].border_color))
+        cr:stroke()
+
         over_drawn_width = width - 2 -- remove 2 for borders
         over_drawn_height = height - 2 -- remove 2 for borders
         border_width = 1
     end
 
-    local angle = 270
-    if data[pbar].vertical then
-        angle = 180
-    end
-
-    -- Draw full gradient
-    if data[pbar].gradient_colors then
-        img:draw_rectangle_gradient(border_width, border_width,
-                                    over_drawn_width, over_drawn_height,
-                                    data[pbar].gradient_colors, angle)
-    else
-        img:draw_rectangle(border_width, border_width,
-                           over_drawn_width, over_drawn_height,
-                           true, data[pbar].color or "red")
-    end
+    cr:rectangle(border_width, border_width,
+                 over_drawn_width, over_drawn_height)
+    cr:set_source(color(data[pbar].color or "#ff0000"))
+    cr:fill()
 
     -- Cover the part that is not set with a rectangle
     if data[pbar].vertical then
-        local rel_height = math.floor(over_drawn_height * (1 - value))
-        img:draw_rectangle(border_width,
-                           border_width,
-                           over_drawn_width,
-                           rel_height,
-                           true, data[pbar].background_color or "#000000aa")
+        local rel_height = over_drawn_height * (1 - value)
+        cr:rectangle(border_width,
+                     border_width,
+                     over_drawn_width,
+                     rel_height)
+        cr:set_source(color(data[pbar].background_color or "#000000aa"))
+        cr:fill()
 
         -- Place smaller pieces over the gradient if ticks are enabled
         if data[pbar].ticks then
@@ -135,76 +118,77 @@ local function update(pbar)
                 local rel_offset = over_drawn_height / 1 - (ticks_size+ticks_gap) * i
 
                 if rel_offset >= rel_height then
-                    img:draw_rectangle(border_width,
-                                       rel_offset,
-                                       over_drawn_width,
-                                       ticks_gap,
-                                       true, data[pbar].background_color or "#000000aa")
+                    cr:rectangle(border_width,
+                                 rel_offset,
+                                 over_drawn_width,
+                                 ticks_gap)
                 end
             end
+            cr:set_source(color(data[pbar].background_color or "#000000aa"))
+            cr:fill()
         end
     else
-        local rel_x = math.ceil(over_drawn_width * value)
-        img:draw_rectangle(border_width + rel_x,
-                           border_width,
-                           over_drawn_width - rel_x,
-                           over_drawn_height,
-                           true, data[pbar].background_color or "#000000aa")
+        local rel_x = over_drawn_width * value
+        cr:rectangle(border_width + rel_x,
+                     border_width,
+                     over_drawn_width - rel_x,
+                     over_drawn_height)
+        cr:set_source(color(data[pbar].background_color or "#000000aa"))
+        cr:fill()
 
         if data[pbar].ticks then
             for i=0, width / (ticks_size+ticks_gap)-border_width do
                 local rel_offset = over_drawn_width / 1 - (ticks_size+ticks_gap) * i
 
                 if rel_offset <= rel_x then
-                    img:draw_rectangle(rel_offset,
-                                       border_width,
-                                       ticks_gap,
-                                       over_drawn_height,
-                                       true, data[pbar].background_color or "#000000aa")
+                    cr:rectangle(rel_offset,
+                                 border_width,
+                                 ticks_gap,
+                                 over_drawn_height)
                 end
             end
+            cr:set_source(color(data[pbar].background_color or "#000000aa"))
+            cr:fill()
         end
     end
+end
 
-    -- Update the image
-    pbar.widget.image = img
+function progressbar.fit(pbar, width, height)
+    return data[pbar].width, data[pbar].height
 end
 
 --- Set the progressbar value.
--- @param pbar The progress bar.
 -- @param value The progress bar value between 0 and 1.
-function set_value(pbar, value)
+function progressbar:set_value(value)
     local value = value or 0
-    local max_value = data[pbar].max_value
-    data[pbar].value = math.min(max_value, math.max(0, value))
-    update(pbar)
-    return pbar
+    local max_value = data[self].max_value
+    data[self].value = math.min(max_value, math.max(0, value))
+    self:emit_signal("widget::updated")
+    return self
 end
 
 --- Set the progressbar height.
--- @param progressbar The progressbar.
 -- @param height The height to set.
-function set_height(progressbar, height)
-    data[progressbar].height = height
-    update(progressbar)
-    return progressbar
+function progressbar:set_height(height)
+    data[self].height = height
+    self:emit_signal("widget::updated")
+    return self
 end
 
 --- Set the progressbar width.
--- @param progressbar The progressbar.
 -- @param width The width to set.
-function set_width(progressbar, width)
-    data[progressbar].width = width
-    update(progressbar)
-    return progressbar
+function progressbar:set_width(width)
+    data[self].width = width
+    self:emit_signal("widget::updated")
+    return self
 end
 
 -- Build properties function
 for _, prop in ipairs(properties) do
-    if not _M["set_" .. prop] then
-        _M["set_" .. prop] = function(pbar, value)
+    if not progressbar["set_" .. prop] then
+        progressbar["set_" .. prop] = function(pbar, value)
             data[pbar][prop] = value
-            update(pbar)
+            pbar:emit_signal("widget::updated")
             return pbar
         end
     end
@@ -214,30 +198,32 @@ end
 -- @param args Standard widget() arguments. You should add width and height
 -- key to set progressbar geometry.
 -- @return A progressbar widget.
-function new(args)
+function progressbar.new(args)
     local args = args or {}
     local width = args.width or 100
     local height = args.height or 20
 
     args.type = "imagebox"
 
-    local pbar = {}
-
-    pbar.widget = capi.widget(args)
-    pbar.widget.resize = false
+    local pbar = base.make_widget()
 
     data[pbar] = { width = width, height = height, value = 0, max_value = 1 }
 
     -- Set methods
     for _, prop in ipairs(properties) do
-        pbar["set_" .. prop] = _M["set_" .. prop]
+        pbar["set_" .. prop] = progressbar["set_" .. prop]
     end
 
-    pbar.layout = args.layout or layout.horizontal.leftright
+    pbar.draw = progressbar.draw
+    pbar.fit = progressbar.fit
 
     return pbar
 end
 
-setmetatable(_M, { __call = function(_, ...) return new(...) end })
+function progressbar.mt:__call(...)
+    return progressbar.new(...)
+end
+
+return setmetatable(progressbar, progressbar.mt)
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
