@@ -1,19 +1,20 @@
 ---------------------------------------------------------------------------
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2009 Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ---------------------------------------------------------------------------
 
 local setmetatable = setmetatable
 
-local capi = { widget = widget }
 local completion = require("awful.completion")
 local util = require("awful.util")
 local prompt = require("awful.prompt")
-local layout = require("awful.widget.layout")
+local widget_base = require("wibox.widget.base")
+local textbox = require("wibox.widget.textbox")
 local type = type
 
-module("awful.widget.prompt")
+-- awful.widget.prompt
+local widgetprompt = { mt = {} }
 
 --- Run method for promptbox.
 -- @param promptbox The promptbox to run.
@@ -23,7 +24,7 @@ local function run(promptbox)
                       function (...)
                           local result = util.spawn(...)
                           if type(result) == "string" then
-                              promptbox.widget.text = result
+                              promptbox.widget:set_text(result)
                           end
                       end,
                       completion.shell,
@@ -31,21 +32,24 @@ local function run(promptbox)
 end
 
 --- Create a prompt widget which will launch a command.
--- @param args Standard widget table arguments, with prompt to change the
--- default prompt.
+-- @param args Arguments table. "prompt" is the prompt to use.
 -- @return A launcher widget.
-function new(args)
+function widgetprompt.new(args)
     local args = args or {}
-    local promptbox = {}
-    args.type = "textbox"
-    promptbox.widget = capi.widget(args)
-    promptbox.widget.ellipsize = "start"
+    local widget = textbox()
+    local promptbox = widget_base.make_widget(widget)
+
+    promptbox.widget = widget
+    promptbox.widget:set_ellipsize("start")
     promptbox.run = run
     promptbox.prompt = args.prompt or "Run: "
-    promptbox.layout = args.layout or layout.horizontal.leftright
     return promptbox
 end
 
-setmetatable(_M, { __call = function (_, ...) return new(...) end })
+function widgetprompt.mt:__call(...)
+    return widgetprompt.new(...)
+end
+
+return setmetatable(widgetprompt, widgetprompt.mt)
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
