@@ -2,11 +2,10 @@
 -- @author Damien Leone &lt;damien.leone@gmail.com&gt;
 -- @author Julien Danjou &lt;julien@danjou.info&gt;
 -- @copyright 2008-2009 Damien Leone, Julien Danjou
--- @release v3.4.14
+-- @release v3.5.1
 ----------------------------------------------------------------------------
 
 -- Grab environment
-local io = io
 local os = os
 local print = print
 local pcall = pcall
@@ -15,34 +14,62 @@ local type = type
 local dofile = dofile
 local setmetatable = setmetatable
 local util = require("awful.util")
-local package = package
+local lgi = require("lgi")
+local cairo = lgi.cairo
+local Pango = lgi.Pango
+local PangoCairo = lgi.PangoCairo
 local capi =
 {
     screen = screen,
-    awesome = awesome,
-    image = image
+    awesome = awesome
 }
 
 --- Theme library.
-module("beautiful")
+local beautiful = { mt = {} }
 
 -- Local data
-local theme
+local theme = {}
+local descs = setmetatable({}, { __mode = 'k' })
+local fonts = setmetatable({}, { __mode = 'v' })
+local active_font
 
--- Checks the entries in the wallpaper_cmd are valid.
--- @param tw A table with wallpaper commands.
--- @return False if there is no way that this can be a valid wallpaper cmd.
-local function wallpaper_is_valid(tw)
-    if type(tw) ~= 'table' then return false end
-    for _, v in pairs(tw) do
-        if type(v) ~= 'string' then return false end
+local function load_font(name)
+    name = name or active_font
+    if name and type(name) ~= "string" and descs[name] then
+        name = descs[name]
     end
-    return (#tw > 0) and true
+    if fonts[name] then
+        return fonts[name]
+    end
+    -- load new font
+    local desc = Pango.FontDescription.from_string(name)
+
+    local ctx = PangoCairo.font_map_get_default():create_context()
+    local layout = Pango.Layout.new(ctx)
+    layout:set_font_description(desc)
+
+    local width, height = layout:get_pixel_size()
+    local font = { name = name, description = desc, height = height }
+    fonts[name] = font
+    descs[desc] = name
+    return font
+end
+
+local function set_font(name)
+    active_font = load_font(name).name
+end
+
+function beautiful.get_font(name)
+    return load_font(name).description
+end
+
+function beautiful.get_font_height(name)
+    return load_font(name).height
 end
 
 --- Init function, should be runned at the beginning of configuration file.
 -- @param path The theme file path.
-function init(path)
+function beautiful.init(path)
     if path then
         local success
         success, theme = pcall(function() return dofile(path) end)
@@ -59,18 +86,7 @@ function init(path)
                 end
             end
 
-            -- setup wallpaper
-            if wallpaper_is_valid(theme.wallpaper_cmd) then
-                for s = 1, capi.screen.count() do
-                    util.spawn(theme.wallpaper_cmd[util.cycle(#theme.wallpaper_cmd, s)], false, s)
-                end
-            else
-                if theme.wallpaper_cmd then
-                    -- There is a wallpaper_cmd, but wallpaper_is_valid doesn't like it
-                    print("W: beautiful: invalid wallpaper_cmd in theme file " .. path)
-                end
-            end
-            if theme.font then capi.awesome.font = theme.font end
+            if theme.font then set_font(theme.font) end
             if theme.fg_normal then capi.awesome.fg = theme.fg_normal end
             if theme.bg_normal then capi.awesome.bg = theme.bg_normal end
         else
@@ -83,10 +99,17 @@ end
 
 --- Get the current theme.
 -- @return The current theme table.
-function get()
+function beautiful.get()
     return theme
 end
 
-setmetatable(_M, { __index = function(t, k) return theme[k] end })
+function beautiful.mt:__index(k)
+    return theme[k]
+end
+
+-- Set the default font
+set_font("sans 8")
+
+return setmetatable(beautiful, beautiful.mt)
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
